local function mainPalette()
	local c = love.graphics.newCanvas(800,600)

	love.graphics.setBlendMode('alpha')
	love.graphics.setColor(255,255,255,255)
	love.graphics.setCanvas(c)
		c:clear()
		love.graphics.setBlendMode('alpha')
		local i = 0
		for y=0,31 do
			for x=0,31 do
				local r = 255*(math.sin(i * math.pi / 128.0)/2.0 + 0.5)
				local g = 0
                local s = math.sin(i * math.pi / 128.0)
				if s > 0.5 then
					s = s - 0.5
					g = (192*s*2.0);
				end
				local b = 128*(math.cos(i * math.pi / 96.0)/2.0 + 0.5)
				local mf = math.floor
				love.graphics.setColor(mf(r),mf(g),mf(b),255)
				love.graphics.point(x+0.5,y+0.5)

				i = i + 1
			end
		end
	love.graphics.setCanvas()
	return c
end

local function firePalette()
	local c = love.graphics.newCanvas(800,600)

	love.graphics.setBlendMode('alpha')
	love.graphics.setColor(255,255,255,255)
	love.graphics.setCanvas(c)
		c:clear()
		love.graphics.setBlendMode('alpha')
		local i = 0
		for y=0,31 do
			for x=0,31 do
				local r = (math.sin(i * math.pi / 128.0)/2.0 + 0.5)
				local g = (math.cos((i+r*32) * math.pi / 128.0)/2.0 + 0.5)
				local b = (math.sin(i * math.pi / 1024.0)/8.0 + 0.125)
				local mf = math.floor
				love.graphics.setColor(mf(255*r),mf(255*g),mf(255*b),255)
				love.graphics.point(x+0.5,y+0.5)

				i = i + 1
			end
		end
	love.graphics.setCanvas()
	return c
end

function love.load()
	balls = {{400,300}, {400,300}, {400,300}}
	pal = mainPalette()
	fire = firePalette()

	love.graphics.setBackgroundColor(0,60,0)
	love.graphics.setColor(255,255,255,255)

	effect = love.graphics.newPixelEffect(([[
		#define NBALLS %d
		extern vec2[NBALLS] balls;

		float dist(vec2 x)
		{
			return sqrt(dot(x, x) + .00001);
		}

		vec4 effect(vec4 color, Image tex, vec2 tc, vec2 pc)
		{
			float p = 0.0;
			float r = 0.0;
			p += sin(dist(pc - balls[0]) / 129.0);
			p += sin(dist(pc - balls[1]) / 149.0);
			p += sin(dist(pc - balls[2]) / 139.0);
			p += NBALLS;
			p /= 2*NBALLS;
			p = p*1024;
			r = p - 32*floor(p/32);
			return vec4(Texel(tex, vec2(r/(32*800), 1 - p/(32*600))).rgb, 1);
		}
	]]):format(#balls))

	effect:send('balls', unpack(balls))

end

whichPal=true
function love.keypressed(k)
	if k == 's' then
		whichPal = not whichPal
	elseif k == "q" or k == "escape" then
		love.event.quit()
	end
end

function love.draw()
	love.graphics.setPixelEffect(effect)

	love.graphics.setBlendMode('premultiplied')
	local p = whichPal and pal or fire
	love.graphics.draw(p, 0, 0)
end

t = 0
function love.update(dt)
	t = t + dt
	local w = pal:getWidth() 
	local h = pal:getHeight()
	local midx = w/2
	local midy = h/2
	balls[1] = {math.sin(t/5) * 400 + midx, math.cos(t/5) * 300 + midy}
	balls[2] = {math.sin(t) * 200 + midx, math.cos(t/3) * 200 + midy}
	balls[3] = {
		math.sin(t) * (110 + math.sin(.01*t) * 110)  + midx,
		math.cos(t) * (110 + math.sin(.01*t) * 110)  + midy,
	}
	effect:send('balls', unpack(balls))
end
